/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.define('DSS.addon.sceneresponder.Panel', {
    extend : 'Ext.grid.Panel',

    config : {
        enableColumnHide : false,
        enableColumnMove : false,
        selType : 'rowmodel',
        viewConfig : {
            forceFit : true
        }
    },

    viewConfig : {
        stripeRows : true
    },

    /** {Object} The event store */
    store: null,
    /** {Object} The window holding the sceneResponderEditor */
    editorWindow: null,
    /** {Object} The currently selected automaton as DSS.addon.sceneresponder.Model (or null if none) */
    curSelectedEvent: null,
    /** {Object} default sorting column */
    defaultSortCol: 'name',
    /** {Object} default sorting direction */
    defaultSortDirection: 'ASC',

    property : 'DSS.json.Property',

    constructor: function(config) {
        var me = this;

        this.initConfig(config);
        this.property = Ext.create(this.property, config);
        this.callParent(arguments);
    },

    initComponent: function() {
        var me = this;

        me.store = Ext.create('Ext.data.Store', {
            storeId : 'sceneresponderStore',
            fields: [
                { name: 'id' },
                { name: 'enabled',  sortType: 'asBoolean' },
                { name: 'name',     sortType: 'asUCText'  },
                { name: 'actions',  sortType: 'asAction'  },
                { name: 'triggers', sortType: 'asTrigger' },
                { name: 'delay',    sortType: 'asTimespan' }
            ],
            proxy : {
                type : 'memory',
                reader : {
                    type : 'json',
                    root : 'entries'
                }
            }
        });

        var editorConfig = me.config;
        editorConfig.title = _('Edit automaton');
        editorConfig.staticControls = [ {
            type : 'text',
            label : _('Name'),
            id : 'name'
        }, {
            type: 'hidden',
            label: 'id',
            id: 'id'
        }];
        editorConfig.tabControls = [{
            tabLabel : 'Ausl&ouml;ser',
            tabDescription : 'W&auml;hlen Sie die Aktivit&auml;t aus, welche als Ausl&ouml;ser dienen soll. Es k&ouml;nnen gleichzeitig mehrere Stimmungen ausgew&auml;hlt werden.',
            content : [{
                type : 'TriggerConfigurator',
                id : 'triggers',
                flex : true
           }, {
                type: 'timespan',
                label: 'Verz&ouml;gerung',
                id: 'delay'
            }]
        }, {
            tabLabel : 'Auszuf&uuml;hrende Aktivit&auml;t',
            tabDescription : 'Geben Sie die Aktivit&auml;ten ein, die in diesem Automaten ausgef&uuml;hrt werden sollen.',
            content : [{
                type : 'ActionConfigurator',
                id : 'actions',
                flex : true
            }]
        }];

        me.editorWindow = Ext.create('DSS.addon.sceneresponder.Editor', editorConfig);
        me.on({
            select: function(model, record, index, eOpts ) {
                var me = this;
                me.curSelectedEvent = record;
                Ext.getCmp('btn-edit').setDisabled(false);
                Ext.getCmp('btn-disable').setDisabled(false);
                Ext.getCmp('btn-delete').setDisabled(false);
                me.setDisabledButton(me.curSelectedEvent.get('enabled'));
            },
            itemdblclick: function() {
                var me = this;
                if (me.curSelectedEvent != null) {
                    me.editSelectedRecord();
                };
            },
            sortchange: function (header, col, direction) {
                var me = this;
                me.defaultSortCol = col.dataIndex;
                me.defaultSortDirection = direction;
            }
        });
        me.editorWindow.on({
            'save' : function(oValue) {
                dss.raiseEvent('scene-responder.config', {
                    actions : 'save',
                    value : Ext.JSON.encode(oValue)
                })
                me.curSelectedEvent = null;
                me.waitAndReload();
            }
        })
        me.callParent(arguments);
        me.loadParameter();
    }, // initComponent

    setDisabledButton: function(val) {
        var btnDisable = Ext.getCmp('btn-disable');
        if (val) {
            btnDisable.setText(_('Enable automaton'));
            btnDisable.setIconCls('icon-enable-action');
            Ext.getCmp('btn-test').setDisabled(true);
        } else {
            btnDisable.setText(_('Disable automaton'));
            btnDisable.setIconCls('icon-disable-action');
            Ext.getCmp('btn-test').setDisabled(false);
        }
    },

    disableButtons: function() {
        Ext.getCmp('btn-edit').setDisabled(true);
        Ext.getCmp('btn-disable').setDisabled(true);
        Ext.getCmp('btn-delete').setDisabled(true);
        Ext.getCmp('btn-test').setDisabled(true);
    },

    editSelectedRecord: function(){
        var oParams = {},
            me = this;
        for (var iIndex=0; iIndex < me.curSelectedEvent.fields.items.length; iIndex++) {
            var sKey = me.curSelectedEvent.fields.items[iIndex].name;
            oParams[sKey] = me.curSelectedEvent.get(sKey);
        }
        me.editorWindow.show();
        me.editorWindow.setValues(oParams);
    },

    tbar: {
        items: [{
            text: _('New automaton'),
            iconCls : 'icon-add-action',
            handler : function() {
                var panel = this.up("gridpanel");
                panel.editorWindow.setValues(null);
                panel.editorWindow.show();
            }
        }, {
            text: _('Edit automaton'),
            id: 'btn-edit',
            iconCls : 'icon-edit-action',
            disabled : true,
            handler : function() {
                var panel = this.up("gridpanel");
                if (panel.curSelectedEvent != null) {
                    panel.editSelectedRecord();
                };
            }
        }, {
            text: _('Disable automaton'),
            id: 'btn-disable',
            iconCls : 'icon-disable-action',
            disabled : true,
            handler : function() {
                var panel = this.up("gridpanel"),
                    disabled = !panel.curSelectedEvent.get('enabled');
                panel.curSelectedEvent.set('enabled', !disabled);
                dss.raiseEvent('scene-responder.config', {
                    actions : (disabled == false? 'enable' : 'disable'),
                    id : panel.curSelectedEvent.get('id')
                });
                panel.waitAndReload();
            }
        }, {
            text : _('Automat testen'),
            id: 'btn-test',
            iconCls : 'icon-test-action',
            disabled : true,
            handler : function() {
                var panel = this.up("gridpanel");
                dss.ajaxAssyncRequestWithoutAnswer("/json/event/raise", {
                    name: 'action_execute',
                    parameter: "path=/scripts/scene-responder/entries/" + panel.curSelectedEvent.get('id')
                });
            }
        }, '->', {
            text: _('Delete automaton'),
            id: 'btn-delete',
            iconCls : 'icon-delete-action',
            disabled : true,
            handler : function() {
                var panel = this.up("gridpanel");
                dss.raiseEvent('scene-responder.config', {
                    actions : 'delete',
                    id : panel.curSelectedEvent.get('id')
                });
                panel.curSelectedEvent = null;
                panel.waitAndReload();
            }
        }]
    },

    columns: [{
        id : 'col-status',
        header : _('Active'),
        dataIndex : 'enabled',
        xtype: 'disabledcolumn',
        resizable : false,
        width : 50,
        listeners: {
            checkchange : function (me, record, recordIndex, checked) {
                var panel = this.up("gridpanel"),
                    model = panel.getSelectionModel();
                dss.raiseEvent('scene-responder.config', {
                    actions : (checked == false? 'enable' : 'disable'),
                    id : record.get('id')
                });
                if (model.isSelected(recordIndex)) {
                    panel.setDisabledButton(checked);
                } else {
                    model.select(recordIndex, true, false);
                }
                panel.waitAndReload();
            }
        }
    }, {
        id : 'col-name',
        header : _('Name'),
        dataIndex : 'name',
        width : 200
    }, {
        id: 'col-trigger',
        header: _('Trigger'),
        dataIndex: 'triggers', // alibi index
        xtype: 'triggercolumn',
        maintainFlex: true,
        flex: 2,
    }, {
        id : 'col-actions',
        header : _('Action Trigger'),
        dataIndex: 'actions', // alibi index
        xtype: 'actioncolumn',
        maintainFlex : true,
        flex : 5,
    }, {
        id: 'col-delay',
        header: _('Delay'),
        dataIndex: 'delay',
        xtype: 'timespancolumn',
        width: 80
    }],

    waitAndReload : function() {
        var me = this;
        var iSubscriptID = Math.floor(Math.random() * 100000000000);
        dss.ajaxSyncRequest('/json/event/subscribe', {
            subscriptionID : iSubscriptID,
            name : 'scene-responder.saved'
        });
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID : iSubscriptID,
            timeout : 60 * 1000
        }, function() {
            me.loadParameter();
            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                subscriptionID : iSubscriptID,
                name : 'scene-responder.saved'
            });
        });
    },

    loadParameter : function(me) {
        if (me == null) {
            me = this;
        }
        me.store.removeAll();
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/scene-responder/entries/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString),
            oResultObject = dss.parsePropertyQueryAnswer(result, [
                {name: 'name', type: 'simple'},
                {name: 'id', type: 'simple'},
                {name: 'enabled', type: 'condition/enabled'},
                {name: 'delay', type: 'simple'},
                {name: 'actions', type: 'action'},
                {name: 'triggers', type: 'trigger'}
            ]);
        if (oResultObject!=null) {
            me.store.add(oResultObject);
            me.store.sort(me.defaultSortCol, me.defaultSortDirection);
            if (me.curSelectedEvent != null && me.curSelectedEvent.get('id') != null) {
                var oToSelect = me.store.findRecord('id', me.curSelectedEvent.get('id'));
                if (oToSelect != null) {
                    me.getSelectionModel().select(oToSelect);
                    me.curSelectedEvent = oToSelect;
                }
            } else {
                me.disableButtons();
            }
        }
    }
});

Ext.define('DSS.addon.sceneresponder.Editor', {
    extend : 'DSS.component.Editor'
});

